/* DIFFH - A diff frontend for output in HTML format.
 * Module:  MAIN
 * Author:  Matt Giuca
 * Created: 5/2/06
 * Updated: 7/2/06
 *
 * Copyright (C) 2006 Matt Giuca <mattgiuca@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Module: MAIN
 * This module is the entry point to the program. It handles command-line
 * arguments and calls the top-level modules. It also defines all global
 * variables.
 */

#include "global.h"
#include "input.h"
#include "output.h"

/* GLOBAL VARIABLE DEFINITIONS */
FILE* outstream;
int tabwidth = DEFAULT_TABWIDTH;
char* custom_css_filename = NULL;
char* css_bodyfont = DEFAULT_CSS_BODYFONT;
char* css_bodyfontsize = DEFAULT_CSS_BODYFONTSIZE;
char* css_bodybackcolor = DEFAULT_CSS_BODYBACKCOLOR;
char* css_color_unchanged = DEFAULT_CSS_COLOR_UNCHANGED;
char* css_color_added = DEFAULT_CSS_COLOR_ADDED;
char* css_color_deleted = DEFAULT_CSS_COLOR_DELETED;
char* css_color_change = DEFAULT_CSS_COLOR_CHANGE;

/* PRIVATE FUNCTION DECLARATIONS */
static void usage(int status);
static void version(void);

/* Function: main
 * The program entry point.
 */
int main(int argc, char** argv)
{
    UDiff* diff;

    outstream = stdout;

    /* Parse command-line options */
    if (argc > 1)
    {
        if (argc == 2 && !strcmp(argv[1], ARG_VERSION))
            version();
        else if (argc == 2 && !strcmp(argv[1], ARG_USAGE))
            usage(EXIT_SUCCESS);
        else
        {
            usage(EXIT_FAILURE);
            return EXIT_FAILURE;
        }
        return 0;
    }

    /* Read the input stream into diff */
    diff = read_diff_file(stdin);
    if (diff == NULL) exit(EXIT_FAILURE);

    /* Process the diff and print out the HTML */
    diff_process(diff);

    return 0;
}

void usage (int status)
{
    if (status != EXIT_SUCCESS)
        fputs("Try `" COMMAND_NAME " --help' for more information.\n",
            stderr);
    else
    {
        fputs ("Usage: " COMMAND_NAME "\n"
                "or:  " COMMAND_NAME " OPTION\n",
                stdout);

        fputs ("\nConvert a unified diff file into HTML format. Pass diff "
            "file as input.\n"
            "Use the -u or -Un argument to diff to generate required "
            "unified diff format.\n"
            "Example usage:\n"
            "    diff -u file1 file2 | " COMMAND_NAME " > diff.html\n"
            "Also accepts output from RCS/CVS diff, if in unified format.\n",
            stdout);
        fputs("\nOPTIONS:\n", stdout);
        fputs("      --help     display this help and exit\n", stdout);
        fputs("      --version  output version information and exit\n",
            stdout);
    }
}

void version(void)
{
    fputs(PROGRAM_NAME " " PROGRAM_VERSION NL
        "Written by " AUTHORS "." NL NL
        GPL_SHORT, stdout);
}
